from typing import Any

import datasets
from mteb.abstasks import AbsTaskRetrieval


class TV2Nordretrieval(AbsTaskRetrieval):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "TV2Nordretrieval",
            "hf_hub_name": "alexandrainst/nordjylland-news-summarization",
            "description": "News Article and corresponding summaries extracted from the Danish newspaper TV2 Nord.",
            "reference": "https://huggingface.co/datasets/alexandrainst/nordjylland-news-summarization",
            "type": "Retrieval",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["da"],
            "main_score": "ndcg_at_10",
            "revision": "80cdb115ec2ef46d4e926b252f2b59af62d6c070",
        }

    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """
        and transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            ds: datasets.Dataset = self.dataset[split]  # type: ignore
            ds = ds.shuffle(seed=42)
            ds = ds.select(range(2048))  # limit the dataset size to make sure the task does not take too long to run
            self.queries[split] = {}
            self.relevant_docs[split] = {}
            self.corpus[split] = {}

            summary = ds["summary"]
            article = ds["text"]

            n = 0
            for summ, art in zip(summary, article):
                self.queries[split][str(n)] = summ
                q_n = n
                n += 1
                if art not in text2id:
                    text2id[art] = n
                    self.corpus[split][str(n)] = {"title": "", "text": art}
                    n += 1
                cor_n = text2id[art]

                self.relevant_docs[split][str(q_n)] = {str(text2id[art]): 1}  # only one correct matches



import pytest
import seb
from seb.registered_tasks.speed import CPUSpeedTask, GPUSpeedTask

from .dummy_model import create_test_model


@pytest.mark.parametrize("model", [create_test_model(), "sentence-transformers/all-MiniLM-L6-v2"])
def test_cpu_speed_task(model: seb.SebModel):
    task = CPUSpeedTask()  # type: ignore
    model = create_test_model()

    result = task.evaluate(model.encoder)

    speed = result.get_main_score()
    assert speed > 0
    assert speed < 1000


@pytest.mark.skip(reason="GPU speed task is not available during CI")
@pytest.mark.parametrize("model", ["sentence-transformers/all-MiniLM-L6-v2"])
def test_gpu_speed_task(model: seb.SebModel):
    task = GPUSpeedTask()  # type: ignore
    model = create_test_model()

    result = task.evaluate(model.encoder)

    speed = result.get_main_score()
    assert speed > 0
    assert speed < 1000



"""Utilities for import coding in python"""

import importlib
import importlib.util
from pathlib import Path
from types import ModuleType
from typing import Optional, Union


def import_file(name: str, loc: Union[str, Path]) -> ModuleType:
    """Import module from a file. Used to load models from a directory.

    Args:
        name: Name of module to load.
        loc: Path to the file.

    Returns:
        The loaded module.
    """
    spec = importlib.util.spec_from_file_location(name, str(loc))
    module = importlib.util.module_from_spec(spec)  # type: ignore[arg-type]
    spec.loader.exec_module(module)  # type: ignore[union-attr]
    return module


def import_code(code_path: Optional[Union[Path, str]]) -> None:
    """Helper to import Python file provided in training commands / commands
    using the config. This makes custom registered functions available.
    """
    if code_path is not None:
        if not Path(code_path).exists():
            raise ValueError(f"Path to Python code not found {Path(code_path)}")
        import_file("python_code", code_path)



from datetime import date
from functools import partial
from typing import Any

from numpy.typing import ArrayLike
from sentence_transformers import SentenceTransformer

from seb.registries import models

from ..interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from ..interfaces.task import Task


class E5Wrapper(Encoder):
    def __init__(self, model_name: str, sep: str = " "):
        self.model_name = model_name
        self.mdl = SentenceTransformer(model_name)
        self.sep = sep

    def encode(  # type: ignore
        self,
        sentences: list[str],
        *,
        task: Task,  # noqa: ARG002
        batch_size: int = 32,
        **kwargs: Any,
    ) -> ArrayLike:
        return self.encode_queries(sentences, batch_size=batch_size, **kwargs)

    def encode_queries(self, queries: list[str], batch_size: int, **kwargs):  # noqa
        sentences = ["query: " + sentence for sentence in queries]
        return self.mdl.encode(sentences, batch_size=batch_size, **kwargs)

    def encode_corpus(self, corpus: list[dict[str, str]], batch_size: int, **kwargs):  # noqa
        if isinstance(corpus, dict):
            sentences = [
                (corpus["title"][i] + self.sep + corpus["text"][i]).strip() if "title" in corpus else corpus["text"][i].strip()  # type: ignore
                for i in range(len(corpus["text"]))  # type: ignore
            ]
        else:
            sentences = [(doc["title"] + self.sep + doc["text"]).strip() if "title" in doc else doc["text"].strip() for doc in corpus]
        sentences = ["passage: " + sentence for sentence in sentences]
        return self.mdl.encode(sentences, batch_size=batch_size, **kwargs)


# English
@models.register("intfloat/e5-small")
def create_e5_small() -> SebModel:
    hf_name = "intfloat/e5-small"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=384,
        release_date=date(2022, 12, 26),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(E5Wrapper, model_name=hf_name)),
        meta=meta,
    )


@models.register("intfloat/e5-base")
def create_e5_base() -> SebModel:
    hf_name = "intfloat/e5-base"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=768,
        release_date=date(2022, 12, 26),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(E5Wrapper, model_name=hf_name)),
        meta=meta,
    )


@models.register("intfloat/e5-large")
def create_e5_large() -> SebModel:
    hf_name = "intfloat/e5-large"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=1024,
        release_date=date(2022, 12, 26),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(E5Wrapper, model_name=hf_name)),
        meta=meta,
    )


# Multilingual
@models.register("intfloat/multilingual-e5-small")
def create_multilingual_e5_small() -> SebModel:
    hf_name = "intfloat/multilingual-e5-small"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=[],
        open_source=True,
        embedding_size=384,
        model_type="BERT",
        release_date=date(2023, 6, 30),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(E5Wrapper, model_name=hf_name)),
        meta=meta,
    )


@models.register("intfloat/multilingual-e5-base")
def create_multilingual_e5_base() -> SebModel:
    hf_name = "intfloat/multilingual-e5-base"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=[],
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2023, 6, 30),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(E5Wrapper, model_name=hf_name)),
        meta=meta,
    )


@models.register("intfloat/multilingual-e5-large")
def create_multilingual_e5_large() -> SebModel:
    hf_name = "intfloat/multilingual-e5-large"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=[],
        open_source=True,
        embedding_size=1024,
        model_type="BERT",
        release_date=date(2023, 6, 30),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(E5Wrapper, model_name=hf_name)),
        meta=meta,
    )
